ifdoc = {}

if not iguana.project then
   error('ifdoc requires Iguana 5.0.13 or later', 3)
end

require('ifdoc.db')
require('ifdoc.defs')
require('ifdoc.log')
require('ifdoc.custprop')
require('ifdoc.page')
require('ifdoc.checkout')
require('ifdoc.run')
require('ifdoc.report')
require('ifdoc.iguana')
ltp = require('ltp.template')

ifdoc.SANDBOX_DIR = 'ifdoc'

ifdoc.VMD = '_ifdoc.vmd'

-- The GUID of the project we're documenting.
ifdoc.PROJECT_GUID = iguana.project.guid

--[[ Add custom properties to this table.  For example,
{
   name = 'Maximum length',
   resultType = 'number', -- should be 'number', 'string', or 'boolean'
   fieldFunc = function(Field) -- executing at "logging-time"
      if Field:isLeaf() then
         return #Field:nodeValue()
      else
         return nil -- N/A
      end
   end,
   finalResultFunc = function(ResultList) -- executed at "document-generation-time"
      return math.max(ResultList)
   end
}
Note that fieldFunc() will only be run on non-empty field.
--]]
ifdoc.customProperties = {}

-- A safe hl7.parse() that should not be overriden.
-- hl7.parse() itself can be overridden, but this
-- should remain untouched.
ifdoc.hl7Parse = hl7.parse

-- Clear any cached HTML files
local Tmp = os.tmpname()
os.execute('del "'..ifdoc.SANDBOX_DIR..'\\*.report.cache.html" 2> '..Tmp)
os.remove(Tmp)

function ifdoc.init(ProjectGuid)
   if ProjectGuid then
      ifdoc.PROJECT_GUID = ProjectGuid
   end
   
   -- Ensure sandbox dir exists.
   local Tmp = os.tmpname()
   os.execute('mkdir "'..ifdoc.SANDBOX_DIR..'" 2> '..Tmp)
   os.remove(Tmp)
   
   ifdoc.db.createTables()
end

function ifdoc.clearLoggedMessages()
   ifdoc.db.emptyTable('Message')
   ifdoc.db.emptyTable('SegmentGroup')
   ifdoc.db.emptyTable('Segment')
   ifdoc.db.emptyTable('Field')
   --ifdoc.db.emptyTable('SubField')
   --ifdoc.db.emptyTable('SubSubField')
   ifdoc.db.emptyTable('CustomProperty')
   ifdoc.db.emptyTable('CustomPropertyDef')
end

function ifdoc.destroy()
   ifdoc.db.dropAllTables()
end

-- Id is optional.  If it is not provided, no check
-- for an existing logged message will be made, and
-- a unique Id will be assigned.
function ifdoc.logInMessage(Msg, Id)
   ifdoc.log.logMessage(Msg, 'in', Id)
end

-- Id is optional.  If it is not provided, no check
-- for an existing logged message will be made, and
-- a unique Id will be assigned.
function ifdoc.logOutMessage(Data, Vmd, Id)
   local Msg = ifdoc.hl7Parse{data=Data, vmd=Vmd}
   ifdoc.log.logMessage(Msg, 'out', Id)
end

function ifdoc.projectDir(ProjectGuid)
   return ifdoc.SANDBOX_DIR..'\\'..ProjectGuid
end