local function trace(A,B,C) end

local function exec(Command) 
   local F = io.popen(Command)
   local R = F:read('*a')
   F:close()
   return R
end

if not iguana.appDir then
   function iguana.appDir()
      return os.getenv('iguana_dir')
   end
end

if not iguana.workingDir then
   function iguana.workingDir()
      local W = exec('cd')
      return W:gsub('\n', '')
   end
end

function executeFossil(FossilDir, Command)
   return exec('cd "'..FossilDir..'" & "'..iguana.appDir()..'\\fossil" '..Command)
end

local function cleanFossil(ProjectDir)
   os.execute('rmdir /s /q '..ifdoc.SANDBOX_DIR..'\\'..ProjectDir)
   os.execute('cd "'..ifdoc.SANDBOX_DIR..'" & mkdir "'..ProjectDir..'"')
end

local function openFossil(FossilDir)
   executeFossil(FossilDir, 'open "'..iguana.workingDir()..'\\vcs_repo.sqlite" --keep')
end

function string.split(s, d)
   local t = {}
   local i = 0
   local f
   local match = '(.-)' .. d .. '()'
   if string.find(s, d) == nil then
      return {s}
   end
   for sub, j in string.gfind(s, match) do
      i = i + 1
      t[i] = sub
      f = j
   end
   if i~= 0 then
      t[i+1]=string.sub(s,f)
   end
   return t
end

local function getLatestMilestone(FossilDir, ProjectGuid)
   -- finfo will get all milestones for a file.
   -- finfo requires posix-style path separators, even on windows.
   local RawMList =
      executeFossil(FossilDir, 'finfo -i "'..ProjectGuid..'/'..'project.prj" tags')
   local MList = RawMList:split('\n')
   trace(MList)
   local M = MList[1]:split('\t')
   local A, B, MName = M[1]:find('__[^_]+__(.*)')
   print('Using latest milestone: "'..MName..'"')
   return M[1]:sub(5)
end

local function revertProject(FossilDir, ProjectGuid, Milestone)
   -- finfo will get the list of files, and their "commit ids",
   -- for the given milestone.
   local RawFList = executeFossil(FossilDir, 'finfo -i "'..Milestone..'" files')
   local FList = RawFList:split('\n')
   FList[#FList] = nil -- remove extra empty item
   trace(FList)
   for i,V in ipairs(FList) do
      local FInfo = V:split('\t')
      local FPath = FInfo[1]
      local FCommitId = FInfo[2]
      executeFossil(FossilDir, 'revert -r '..FCommitId..' "'..FPath..'"')
   end
end

function ifdoc.checkout(ProjectGuid)
   cleanFossil(ProjectGuid)
   local FossilDir = ifdoc.projectDir(ProjectGuid)
   openFossil(FossilDir)
   local Milestone = getLatestMilestone(FossilDir, ProjectGuid)
   local Vmds = revertProject(FossilDir, ProjectGuid, Milestone)
   -- Move VMDs into a subdirectory of this project's 'other' directory.
   -- (this will move all "other" files as well).
   local VirtualOtherDir = iguana.project.root()..'/other/'..ProjectGuid
   exec('rmdir /s/q "'..VirtualOtherDir..'"')
   os.rename(FossilDir..'\\other', VirtualOtherDir)
end