ifdoc.db = {}

-- These tables are also represented in ifdoc.vmd
local TABLE_DEFINITIONS = {
   'CREATE TABLE IF NOT EXISTS MessageDef (Id TEXT(32) NOT NULL ,Name TEXT,Structure TEXT,Direction TEXT(3) NOT NULL ,PRIMARY KEY (Id,Name,Structure,Direction));',
   'CREATE TABLE IF NOT EXISTS SegmentGroupDef (Id TEXT(32) NOT NULL ,ParentId TEXT(32),Name TEXT,GrammarIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS SegmentDef (Id TEXT(32) NOT NULL ,ParentId TEXT(32),Name TEXT,GrammarIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS FieldDef (Id TEXT(32) NOT NULL ,SegmentDefId TEXT(32),Name TEXT,GrammarIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS SubFieldDef (Id TEXT(32) NOT NULL ,FieldDefId TEXT(32),Name TEXT,GrammarIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS SubSubFieldDef (Id TEXT(32) NOT NULL ,SubFieldDefId TEXT(32),Name TEXT,GrammarIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS Message (Id TEXT(32) NOT NULL ,MessageDefId TEXT(32),Direction TEXT(3) NOT NULL ,PRIMARY KEY (Id,MessageDefId,Direction));',
   'CREATE TABLE IF NOT EXISTS SegmentGroup (Id TEXT(32) NOT NULL ,ParentId TEXT(32),SegmentGroupDefId TEXT(32),RepeatIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS Segment (Id TEXT(32) NOT NULL ,ParentId TEXT(32),SegmentDefId TEXT(32),RepeatIndex INT4,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS Field (Id TEXT(32) NOT NULL ,SegmentId TEXT(32),FieldDefId TEXT(32),RepeatIndex INT4,Value TEXT,PRIMARY KEY (Id));',
   --'CREATE TABLE IF NOT EXISTS SubField (Id TEXT(32) NOT NULL ,FieldId TEXT(32),SubFieldDefId TEXT(32),Value TEXT,PRIMARY KEY (Id));',
   --'CREATE TABLE IF NOT EXISTS SubSubField (Id TEXT(32) NOT NULL ,SubFieldId TEXT(32),SubSubFieldDefId TEXT(32),Value TEXT,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS CustomPropertyDef (Id TEXT(32) NOT NULL ,Name TEXT,PRIMARY KEY (Id));',
   'CREATE TABLE IF NOT EXISTS CustomProperty (CustomPropertyDefId TEXT(32),FieldDefId TEXT(32),Result TEXT);',
   'CREATE TABLE IF NOT EXISTS InProgress (DataSize INT4);'
}

local DROP_COMMANDS = {
   'DROP TABLE IF EXISTS MessageDef',
   'DROP TABLE IF EXISTS SegmentGroupDef',
   'DROP TABLE IF EXISTS SegmentDef',
   'DROP TABLE IF EXISTS FieldDef',
   'DROP TABLE IF EXISTS SubFieldDef',
   'DROP TABLE IF EXISTS SubSubFieldDef',
   'DROP TABLE IF EXISTS Message',
   'DROP TABLE IF EXISTS SegmentGroup',
   'DROP TABLE IF EXISTS Segment',
   'DROP TABLE IF EXISTS Field',
   'DROP TABLE IF EXISTS SubField',
   'DROP TABLE IF EXISTS SubSubField',
   'DROP TABLE IF EXISTS CustomPropertyDef',
   'DROP TABLE IF EXISTS CustomProperty'
}

function ifdoc.db.newGuid()
   return util.guid(128)
end

function ifdoc.db.escape(Value) 
   if type(Value) ~= 'string' then Value = tostring(Value) end
   Value = Value:gsub('"', '""') 
   Value = Value:gsub("'", "''") 
   return Value 
end

function ifdoc.db.execute(Sql)
   return db.execute{
      api=db.SQLITE,
      name=ifdoc.SANDBOX_DIR..'\\'..ifdoc.PROJECT_GUID..'.db',
      sql=Sql,
      live=true
   }
end

local dbMerge -- forward declaration

function ifdoc.db.merge(Tables)
   return dbMerge{
   --return db.merge{
      api=db.SQLITE,
      name=ifdoc.SANDBOX_DIR..'\\'..ifdoc.PROJECT_GUID..'.db',
      data=Tables,
      live=true
   }
end

function ifdoc.db.createTables()
   for i,Def in ipairs(TABLE_DEFINITIONS) do
      ifdoc.db.execute(Def)
   end
end

function ifdoc.db.dropAllTables()
   for i,Drop in ipairs(DROP_COMMANDS) do
      ifdoc.db.execute(Drop)
   end
end

function ifdoc.db.emptyTable(TableName)
   ifdoc.db.execute('DELETE FROM '..TableName)
end

-- Below is a pure Lua implementation of db.merge(),
-- taken from http://wiki.interfaceware.com/700.html
-- and altered to support sqlite.

local function StringValue(N, R, Esc)
   if N:isNull() then
      R[#R+1] = '""'
   elseif N:S():match('0x') then
      R[#R+1] = tostring(N)
   else
      R[#R+1] = '"'
      R[#R+1] = ifdoc.db.escape(tostring(N))
      R[#R+1] = '"' 
   end
end

local function DateValue(N, R)
   if N:isNull() then
      R[#R+1] = 'NULL'
   else
      R[#R+1] = '"'
      R[#R+1] = tostring(N)
      R[#R+1] = '"' 
   end
end

local function DoubleValue(N, R)
   if N:isNull() then
      R[#R+1] = 'NULL'
   else
      R[#R+1] = tostring(N)
   end      
end

local function IntegerValue(N, R)
   if N:isNull() then
      R[#R+1] = 'NULL'
   else
      R[#R+1] = tostring(N)
   end         
end

local ValueTable={
   string=StringValue,   
   datetime=DateValue,
   double=DoubleValue,
   integer=IntegerValue
}

local function MergeTableValue(N, R, Esc)
   ValueTable[N:nodeType()](N, R, Esc)
end

local function MergeTableRow(T, R, Esc)
   R[#R+1] = '('
   for i=1, #T do
      if i ~= 1 then
         R[#R+1] = ','
      end
      R[#R+1] = T[i]:nodeName()
   end
   R[#R+1] = ') VALUES('
   for i=1, #T do
      if i ~= 1 then
         R[#R+1] = ','
      end
      MergeTableValue(T[i], R, Esc)
   end
   
   R[#R+1] = '); '
end

local function MergeTable(T, R, output, Esc)
   for i=1, #T do
      R[#R+1] = 'REPLACE INTO '
      R[#R+1] = T:nodeName()
      MergeTableRow(T[i], R, Esc)
      output[#output+1] = table.concat(R)
      R={}
   end
end

local function BackSlashEscape(Value) 
   Value = Value:gsub('"', '\\"')
   Value = Value:gsub("'", "\\'")
   Value = Value:gsub("\\", "\\\\")
   return Value
end

local function GetTableList(T)
   local R ={}
   for i=1, #T do
      R[i] = T[i]:nodeName()   
   end
   return R
end

local function CheckTable(T,Usage)
   if type(T) ~= 'table' then
      error(Usage,3)
   end
end

local function CheckParam(T, List, Usage)
   for i=1, #List do
      if not T[List[i]] then error('Missing parameter "'..List[i]..'"\n'..Usage, 3) end
   end

end

local Usage = [[
Merges table records into the database.  This is a special 
implementation - NOT the default implementation of db.merge.
It only supports MySQL

Returns: nothing.
Accepts a table with the required entries:
   'name'     - database name/address.
   'user'     - user name 
   'password' - password 
   'data'     - set to a node table tree created using db.tables
These additional optional entries exist:
   'live'        - if true, the merge will be executed in the editor,
   'table_order' - optionally give an array table names in the order you would like to insert them 

e.g. db.merge{name='test@localhost', user='root', password='secret', data=Out}
]]

local function MakeSql(T, TableOrder)
   if not TableOrder then
      TableOrder=GetTableList(T)   
   end
   local R={}
   local output = {'BEGIN TRANSACTION;'}
   for t=1, #TableOrder do
      local TableName = TableOrder[t]
      MergeTable(T[TableName], R, output, Esc)
      R={}
   end
   output[#output + 1] = 'COMMIT;'

   return output
end

function dbMerge(P)
   CheckTable(P, Usage)
   CheckParam(P, {'name','data','api'}, Usage)
   if P.api ~= db.SQLITE then error('Sorry only db.SQLITE supported currently.') end
   if type(P.data) ~= 'userdata' or P.data:nodeType() ~= 'table_collection' then
      error('The data parameter must be a table collection as produced from db.tables{}\n'..Usage,2)
   end
   
   local Sql = MakeSql(P.data, P.table_order)
   
   for i=1,#Sql do
      db.execute{api=P.api, name=P.name, user=P.user, password=P.password, sql=Sql[i], live=P.live}
   end
   if P.live == nil or P.live == false then      
      return "Not executed in editor.  To execute set live=true in input parameters", Sql
   else
      return "Executed in editor since live=true.", Sql
   end  
end

function node.addRow(T, Id)
   local R = T[#T+1]
   if Id then
      R.Id = Id
   end
   return R
end