ifdoc.defs = {}

local addChildDefs

local function messageStructure(Node)
   local S = ''
   for i=1,#Node do
      local Child = Node[i]
      local ChildType = Child:nodeType()
      if ChildType == 'segment' then
         S = S..Child:nodeName()..' '
      elseif ChildType == 'segment_repeated' then
         S = S..'{'..Child:nodeName()..'} '
      elseif ChildType == 'segment_group' then
         S = S..messageStructure(Child[1])..' '
      elseif ChildType == 'segment_group_repeated' then
         S = S..'{'..messageStructure(Child[1])..'} '
      end
   end
   if #S > 1 then
      return S:sub(1, #S-1) -- remove trailing space
   else
      return S
   end
end

--[[
local function addSubSubFieldDef(
      Defs, SubSubField, SubFieldDefId, GrammarIndex)
   local SubSubFieldDef = Defs.SubSubFieldDef:addRow(ifdoc.db.newGuid())
   SubSubFieldDef.SubFieldDefId = SubFieldDefId
   SubSubFieldDef.Name = SubSubField:nodeName()
   SubSubFieldDef.GrammarIndex = GrammarIndex
   return SubSubFieldDef
end

local function addSubFieldDef(
      Defs, SubField, FieldDefId, GrammarIndex)
   local SubFieldDefId = ifdoc.db.newGuid()
   local SubFieldDef = Defs.SubFieldDef:addRow(SubFieldDefId)
   SubFieldDef.FieldDefId = FieldDefId
   SubFieldDef.Name = SubField:nodeName()
   SubFieldDef.GrammarIndex = GrammarIndex
   
   for i=1,#SubField do
      addSubSubFieldDef(Defs, SubField[i], SubFieldDefId, i)
   end
   
   return SubFieldDef
end
--]]

local function addFieldDef(
      Defs, Field, SegmentDefId, GrammarIndex)
   local FieldDefId = ifdoc.db.newGuid()
   local FieldDef = Defs.FieldDef:addRow(FieldDefId)
   FieldDef.SegmentDefId = SegmentDefId
   FieldDef.Name = Field:nodeName()
   FieldDef.GrammarIndex = GrammarIndex
   
   --[[for i=1,#Field do
      addSubFieldDef(Defs, Field[i], FieldDefId, i)
   end]]
   
   return FieldDef
end

local function addSegmentDef(
      Defs, Segment, ParentDefId, GrammarIndex)
   local SegmentDefId = ifdoc.db.newGuid()
   local SegmentDef = Defs.SegmentDef:addRow(SegmentDefId)
   SegmentDef.ParentId = ParentDefId
   SegmentDef.Name = Segment:nodeName()
   SegmentDef.GrammarIndex = GrammarIndex
   
   for i=1,#Segment do
      local Child = Segment[i]
      local ChildType = Child:nodeType()
      if ChildType == 'field' then
         addFieldDef(Defs, Child, SegmentDefId, i)
      elseif ChildType == 'field_repeated' then
         addFieldDef(Defs, Child[1], SegmentDefId, i)
      end
   end
   
   return SegmentDef
end

local function addSegmentGroupDef(
      Defs, SegmentGroup, ParentDefId, GrammarIndex)
   local SegmentGroupDefId = ifdoc.db.newGuid()
   local SegmentGroupDef = Defs.SegmentGroupDef:addRow(SegmentGroupDefId)
   SegmentGroupDef.ParentId = ParentDefId
   SegmentGroupDef.Name = SegmentGroup:nodeName()
   SegmentGroupDef.GrammarIndex = GrammarIndex
   
   addChildDefs(Defs, SegmentGroup, SegmentGroupDefId)
   
   return SegmentGroupDef
end

function addChildDefs(Defs, Parent, ParentDefId)
   for i=1,#Parent do
      local Child = Parent[i]
      local ChildType = Child:nodeType()
      if ChildType == 'segment' then
         addSegmentDef(Defs, Child, ParentDefId, i)
      elseif ChildType == 'segment_repeated' then
         addSegmentDef(Defs, Child[1], ParentDefId, i)
      elseif ChildType == 'segment_group' then
         addSegmentGroupDef(Defs, Child, ParentDefId, i)
      elseif ChildType == 'segment_group_repeated' then
         addSegmentGroupDef(Defs, Child[1], ParentDefId, i)
      end
   end
end

local function addMessageDef(Msg, Structure, Direction)
   local Defs = db.tables{vmd=ifdoc.VMD, name='Definitions'}
   
   local MessageDefGuid = ifdoc.db.newGuid()
   local MessageDef = Defs.MessageDef:addRow(MessageDefGuid)
   MessageDef.Name = Msg:nodeName()
   MessageDef.Structure = Structure
   MessageDef.Direction = Direction
   
   addChildDefs(Defs, Msg, MessageDefGuid)
   
   ifdoc.db.merge(Defs)
   
   return MessageDefGuid
end

function ifdoc.defs.messageDefId(Msg, Direction)
   local Structure = messageStructure(Msg)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM MessageDef WHERE Name = \''
      ..ifdoc.db.escape(Msg:nodeName())..'\' AND Structure = \''
      ..ifdoc.db.escape(Structure)..'\' AND Direction = \''
      ..ifdoc.db.escape(Direction)..'\'')
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   else
      return addMessageDef(Msg, Structure, Direction)
   end
end

function ifdoc.defs.segmentGroupDefId(ParentId, Index)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM SegmentGroupDef '..
      'WHERE ParentId = \''..ifdoc.db.escape(ParentId)..'\' '..
      'AND GrammarIndex = '..Index)
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   end
   error('Segment group definition not found', 2)
end

function ifdoc.defs.segmentDefId(ParentId, Index)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM SegmentDef '..
      'WHERE ParentId = \''..ifdoc.db.escape(ParentId)..'\' '..
      'AND GrammarIndex = '..Index)
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   end
   error('Segment definition not found', 2)
end

function ifdoc.defs.fieldDefId(SegmentDefId, Index)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM FieldDef '..
      'WHERE SegmentDefId = \''..ifdoc.db.escape(SegmentDefId)..'\' '..
      'AND GrammarIndex = '..Index)
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   end
   error('Field definition not found', 2)
end

--[[
function ifdoc.defs.subFieldDefId(FieldDefId, Index)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM SubFieldDef '..
      'WHERE FieldDefId = \''..ifdoc.db.escape(FieldDefId)..'\' '..
      'AND GrammarIndex = '..Index)
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   end
   error('Sub-field definition not found', 2)
end

function ifdoc.defs.subSubFieldDefId(SubFieldDefId, Index)
   local RS = ifdoc.db.execute(
      'SELECT Id FROM SubSubFieldDef '..
      'WHERE SubFieldDefId = \''..ifdoc.db.escape(SubFieldDefId)..'\' '..
      'AND GrammarIndex = '..Index)
   if #RS > 0 then
      return RS[1].Id:nodeValue()
   end
   error('Sub-sub-field definition not found', 2)
end
--]]
