ifdoc.log = {}

--[[
local function logSubSubField(
      Out, SubFieldLogId, SubSubField, SubSubFieldDefId)
   local SubSubFieldLog = Out.SubSubField:addRow(ifdoc.db.newGuid())
   SubSubFieldLog.SubFieldLogId = SubFieldLogId
   SubSubFieldLog.SubSubFieldDefId = SubSubFieldDefId
   SubSubFieldLog.Value = SubSubField:nodeValue()
   
   return SubSubFieldLog
end

local function logSubField(Out, FieldLogId, SubField, SubFieldDefId)
   local SubFieldLogId = ifdoc.db.newGuid()
   local SubFieldLog = Out.SubField:addRow(SubFieldLogId)
   SubFieldLog.FieldLogId = FieldLogId
   SubFieldLog.SubFieldDefId = SubFieldDefId
   
   if SubField:isLeaf() then
      SubFieldLog.Value = SubField:nodeValue()
   else
      for i=1,#SubField do
         local SubSubField = SubField[i]
         if not SubSubField:isNull() then
            local SubSubFieldDefId =
               ifdoc.defs.subSubFieldDefId(SubFieldDefId, i)
            logSubSubField(
               Out, SubFieldLogId, SubSubField, SubSubFieldDefId)
         end
      end
   end
   
   return SubFieldLog
end
--]]

local function logField(Out, SegmentLogId, Field, FieldDefId, RepeatIndex, Segment, Index)
   local FieldLogId = ifdoc.db.newGuid()
   local FieldLog = Out.Field:addRow(FieldLogId)
   FieldLog.SegmentId = SegmentLogId
   FieldLog.FieldDefId = FieldDefId
   FieldLog.RepeatIndex = RepeatIndex
   
   --[[if Field:isLeaf() then
      FieldLog.Value = Field:nodeValue()
   else
      for i=1,#Field do
         local SubField = Field[i]
         if not SubField:isNull() then
            local SubFieldDefId = ifdoc.defs.subFieldDefId(FieldDefId, i)
            logSubField(Out, FieldLogId, SubField, SubFieldDefId)
         end
      end
   end]]
   
   --FieldLog.Value = tostring(Field)
   
   ifdoc.custprop.computeForField(Out.CustomProperty, Field, FieldDefId, Segment, Index)
   
   return FieldLog
end

local function logSegment(
      Out, ParentLogId, Segment, SegmentDefId, RepeatIndex)
   local SegmentLogId = ifdoc.db.newGuid()
   local SegmentLog = Out.Segment:addRow(SegmentLogId)
   SegmentLog.ParentId = ParentLogId
   SegmentLog.SegmentDefId = SegmentDefId
   SegmentLog.RepeatIndex = RepeatIndex
   
   for i=1,#Segment do
      local Child = Segment[i]
      if not Child:isNull() then
         local ChildType = Child:nodeType()
         local FieldDefId = ifdoc.defs.fieldDefId(SegmentDefId, i)
         if ChildType == 'field' then
            logField(Out, SegmentLogId, Child, FieldDefId, 1, Segment, i)
         elseif ChildType == 'field_repeated' then
            for j=1,#Child do
               logField(Out, MessageLogId, Child[j], FieldDefId, j, Segment, i)
            end
         end
      end
   end
   
   return SegmentLog
end

local logChildren

local function logSegmentGroup(
      Out, ParentLogId, SegmentGroup, SegmentGroupDefId, RepeatIndex)
   local SegmentGroupLogId = ifdoc.db.newGuid()
   local SegmentGroupLog = Out.SegmentGroup:addRow(SegmentGroupLogId)
   SegmentGroupLog.ParentId = ParentLogId
   SegmentGroupLog.SegmentGroupDefId = SegmentGroupDefId
   SegmentGroupLog.RepeatIndex = RepeatIndex
   
   logChildren(Out, SegmentGroupLogId, SegmentGroup, SegmentGroupDefId)
   
   return SegmentGroupLog
end

function logChildren(Out, ParentLogId, Parent, ParentDefId)
   for i=1,#Parent do
      local Child = Parent[i]
      if not Child:isNull() then
         local ChildType = Child:nodeType()
         if ChildType == 'segment' then
            local SegmentDefId =ifdoc.defs.segmentDefId(ParentDefId, i)
            logSegment(Out, ParentLogId, Child, SegmentDefId, 1)
         elseif ChildType == 'segment_repeated' then
            local SegmentDefId =ifdoc.defs.segmentDefId(ParentDefId, i)
            for j=1,#Child do
               logSegment(Out, ParentLogId, Child[j], SegmentDefId, j)
            end
         elseif ChildType == 'segment_group' then
            local SegmentGroupDefId = ifdoc.defs.segmentGroupDefId(ParentDefId, i)
            logSegmentGroup(Out, ParentLogId, Child, SegmentGroupDefId, 1)
         elseif ChildType == 'segment_group_repeated' then
            local SegmentGroupDefId = ifdoc.defs.segmentGroupDefId(ParentDefId, i)
            for j=1,#Child do
               logSegmentGroup(Out, ParentLogId, Child[j], SegmentGroupDefId, j)
            end
         end
      end
   end
end

local function messageIsLogged(Id, Direction)
   return #
   ifdoc.db.execute('SELECT Id FROM Message WHERE Id = \''..
      ifdoc.db.escape(Id)..
      '\' AND Direction = \''..ifdoc.db.escape(Direction)..'\'')
   > 0
end

function ifdoc.log.logMessage(Msg, Direction, Id)
   local S = os.time()
   local Out = db.tables{vmd=ifdoc.VMD, name='Data'}
   
   -- Abort if this message has already been logged.
   if Id and messageIsLogged(Id, Direction) then return end
   
   local MessageDefId = ifdoc.defs.messageDefId(Msg, Direction)
   
   ifdoc.custprop.ensureDefsExist()
   
   local MessageLogId = Id or ifdoc.db.newGuid()
   Out.Message[1].Id = MessageLogId
   Out.Message[1].MessageDefId = MessageDefId
   Out.Message[1].Direction = Direction
   
   logChildren(Out, MessageLogId, Msg, MessageDefId)
   
   local SM = os.time()
   ifdoc.db.merge(Out)
   trackTime('db.merge(Out)', SM)
   trackTime('logMessage()', S)
end
