ifdoc.report = {}

local function isTrue(RS)
   return RS[1][1]:nodeValue() == '1'
end

local function isFieldRepeating(FieldDefId)
   local S = os.time()
   local RS = ifdoc.db.execute(
      'SELECT COUNT(F.Id) > 0 FROM Field F WHERE F.FieldDefId =\''..
      ifdoc.db.escape(FieldDefId)..'\' AND F.RepeatIndex > 1')
   trackTime('isFieldRepeating()', S)
   return isTrue(RS)
end

local function isFieldOptional(FieldDefId, SegmentCount)
   local S = os.time()

   local RS = ifdoc.db.execute(
      'SELECT COUNT(DISTINCT S.Id) < '..SegmentCount..
      ' FROM Segment S INNER JOIN Field F ON S.Id = F.SegmentId WHERE '..
      'F.FieldDefId =\''..ifdoc.db.escape(FieldDefId)..'\'')
   
   trackTime('isFieldOptional()', S)
   return isTrue(RS)
end

local function getFields(SegmentDefId, SegmentCount)
   local SFS = os.time()
   
   local RS = ifdoc.db.execute(
      'SELECT FD.* FROM FieldDef FD WHERE FD.SegmentDefId = \''..
      ifdoc.db.escape(SegmentDefId)..'\' AND EXISTS ('..
      '   SELECT F.Id FROM Field F WHERE F.FieldDefId = FD.Id)')
   
   trackTime('SELECT FD.* From FieldDef ...', SFS)
   
   local FList = {}
   for i=1,#RS do
      local R = RS[i]
      local FieldDefId = R.Id:nodeValue()
      local F = {
         --id = FieldDefId,
         name = R.Name:nodeValue(),
         repeats = isFieldRepeating(FieldDefId),
         optional = isFieldOptional(FieldDefId, SegmentCount),
         index = R.GrammarIndex:nodeValue()
      }
      local CPS = os.time()
      F.customProperties = ifdoc.custprop.calculateFinalResult(FieldDefId)
      trackTime('calculateFinalResult()', CPS)
      FList[#FList+1] = F
   end
   
   trackTime('getFields()', SFS)
   
   return FList
end

local function isSegmentGroupRepeating(SegmentGroupDefId)
   local S = os.time()
   local RS = ifdoc.db.execute(
      'SELECT COUNT(SG.Id) > 0 FROM SegmentGroup SG WHERE '..
      'SG.SegmentGroupDefId =\''..ifdoc.db.escape(SegmentGroupDefId)..
      '\' AND SG.RepeatIndex > 1')
   trackTime('isSegmentGroupRepeating()', S)
   return isTrue(RS)
end

local function isSegmentGroupOptional(SegmentGroupDefId, ParentTable, ParentCount)
   local S = os.time()
   
   local RS = ifdoc.db.execute(
      'SELECT COUNT(DISTINCT P.Id) < '..ParentCount..
      ' FROM '..ParentTable..' P INNER JOIN SegmentGroup SG ON P.Id = SG.ParentId WHERE '..
      'SG.SegmentGroupDefId =\''..ifdoc.db.escape(SegmentGroupDefId)..'\'')
   
   trackTime('isSegmentGroupOptional()', S)
   return isTrue(RS)
end

local function isSegmentRepeating(SegmentDefId)
   local S = os.time()
   local RS = ifdoc.db.execute(
      'SELECT COUNT(S.Id) > 0 FROM Segment S WHERE '..
      'S.SegmentDefId =\''..ifdoc.db.escape(SegmentDefId)..
      '\' AND S.RepeatIndex > 1')
   trackTime('isSegmentRepeating()', S)
   return isTrue(RS)
end

local function isSegmentOptional(SegmentDefId, ParentTable, ParentCount)
   local S = os.time()

   local RS = ifdoc.db.execute(
      'SELECT COUNT(DISTINCT P.Id) < '..ParentCount..
      ' FROM '..ParentTable..' P INNER JOIN Segment S ON P.Id = S.ParentId WHERE '..
      'S.SegmentDefId =\''..ifdoc.db.escape(SegmentDefId)..'\'')
   
   trackTime('isSegmentOptional()', S)
   return isTrue(RS)
end

local getSegments
function getSegments(ParentDefId, ParentTable, SegmentList, ParentCount)
   local SSS = os.time()
   
   -- Select segments
   local RS = ifdoc.db.execute(
      'SELECT SD.*, COUNT(S.Id) AS LogCount '..
      'FROM SegmentDef SD INNER JOIN Segment S '..
      'ON S.SegmentDefId = SD.Id WHERE SD.ParentId = \''..
      ifdoc.db.escape(ParentDefId)..'\' GROUP BY SD.Id')
   
   trackTime('SELECT SD.* FROM SegmentDef ...', SSS)
   
   for r=1,#RS do
      local R = RS[r]
      local Id = R.Id:nodeValue()
      local i = tonumber(R.GrammarIndex:nodeValue())
      local SegmentCount = tonumber(R.LogCount:nodeValue())
      local S = {
         id = Id,
         name = R.Name:nodeValue(),
         repeats = isSegmentRepeating(Id),
         optional = isSegmentOptional(Id, ParentTable, ParentCount),
         fields = getFields(Id, SegmentCount)
      }
      SegmentList[i] = S
   end
   
   local SSGS = os.time()
   
   -- Select segment groups
   local RS = ifdoc.db.execute(
      'SELECT SGD.*, COUNT(SG.Id) AS LogCount '..
      'FROM SegmentGroupDef SGD INNER JOIN SegmentGroup SG '..
      'ON SG.SegmentGroupDefId = SGD.Id WHERE SGD.ParentId = \''..
      ifdoc.db.escape(ParentDefId)..'\' GROUP BY SGD.Id')
   
   trackTime('SELECT SGD.* FROM SegmentGroupDef ...', SSGS)
   
   for r=1,#RS do
      local R = RS[r]
      local Id = R.Id:nodeValue()
      local i = tonumber(R.GrammarIndex:nodeValue())
      local SegmentGroupCount = tonumber(R.LogCount:nodeValue())
      local SG = {
         id = Id,
         name = R.Name:nodeValue(),
         repeats = isSegmentGroupRepeating(Id),
         optional = isSegmentGroupOptional(Id, ParentTable, ParentCount),
         isGroup = true,
         segments = {}
      }
      getSegments(Id, 'SegmentGroup', SG.segments, SegmentGroupCount)
      SegmentList[i] = SG
   end
   
   trackTime('getSegments()', SSS)
end

local function getDirectionalMessages(Direction)
   local RS = ifdoc.db.execute(
      'SELECT MD.*, COUNT(M.Id) As MessageCount FROM MessageDef MD '..
      'INNER JOIN Message M ON MD.Id = M.MessageDefId '..
      'WHERE MD.Direction = \''..ifdoc.db.escape(Direction)..
      '\' GROUP BY MD.Id ORDER BY MessageCount DESC')
   
   local MList = {}
   local TotalCount = 0
   
   for i=1,#RS do
      local R = RS[i]
      local MessageDefId = R.Id:nodeValue()
      local MessageCount = tonumber(R.MessageCount:nodeValue())
      TotalCount = TotalCount + MessageCount
      local M = {
         id = MessageDefId,
         name = R.Name:nodeValue(), 
         --structure = R.Structure:nodeValue(),
         count = MessageCount,
         segments = {}
      }
      getSegments(MessageDefId, 'Message', M.segments, MessageCount)
      MList[#MList+1] = M
   end
   
   return MList, TotalCount
end

function math.round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function ifdoc.report.getMessages()
   local M = {}
   M.inbound, M.inboundCount = getDirectionalMessages('in')
   M.outbound, M.outboundCount = getDirectionalMessages('out')
   
   for i,InM in ipairs(M.inbound) do
      InM.percent = math.round(100*InM.count/M.inboundCount, 1)
   end
   
   for i,OutM in ipairs(M.outbound) do
      OutM.percent = math.round(100*OutM.count/M.outboundCount, 1)
   end
   
   return M
end

function ifdoc.report.getCustomProperties()
   local RS = ifdoc.db.execute(
      'SELECT Name FROM CustomPropertyDef')
   local CP = {}
   for i=1,#RS do
      CP[#CP+1] = RS[i].Name:nodeValue()
   end
   return CP
end
